package au.com.infomedix.modeshapeloadtest;

import org.jboss.cache.transaction.JBossStandaloneJTAManagerLookup;
import org.modeshape.common.collection.Problem;
import org.modeshape.jcr.JcrConfiguration;
import org.modeshape.jcr.JcrEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;
import java.util.concurrent.TimeUnit;
import javax.jcr.Binary;
import javax.jcr.InvalidItemStateException;
import javax.jcr.Node;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.transaction.TransactionManager;


/**
 * Hello world!
 *
 */
public class App
{
    private final static Logger log = LoggerFactory.getLogger(App.class);

    public static final String CONTENT_TYPE        = "inf:content";
    public static final String CONTENT_DATA_TYPE   = "nt:file";
    public static final String RESOURCE_DATA_TYPE  = "nt:resource";
    public static final String DOCUMENT_TYPE       = "inf:document";
    public static final String DOCUMENT_GROUP_TYPE = "inf:documentGroup";
    static JcrEngine           engine              = null;

    private static Repository repository;

    public static void main(String[] args)
    {
        try
        {
            JcrConfiguration configuration = new JcrConfiguration();
            configuration.loadFrom(App.class.getResourceAsStream("/modeshape-config.xml"));

            // Now create the JCR engine ...
            engine = configuration.build();
            engine.start();

            if(engine.getProblems().hasProblems())
            {
                for(Problem problem : engine.getProblems())
                {
                    System.err.println(problem.getMessageString());
                }

                throw new RuntimeException("Could not start due to problems");
            }
        }
        catch(IOException e)
        {
            log.error("Failed to open modeshape config file", e);
        }
        catch(SAXException e)
        {
            log.error("Couldn't parse modeshape config file", e);
        }

        Session session = null;

        try
        {
            repository = engine.getRepository("DocumentStore");
        }
        catch(RepositoryException e)
        {
            log.error("Error getting repository setup", e);
        }
        finally
        {
            if(session != null)
            {
                session.logout();
            }
        }

        ArrayList<Thread> threads = new ArrayList<Thread>();

        for(int t = 0; t < 4; t++)
        {
            final int tt = t;
            threads.add(new Thread()
                {
                    public void run()
                    {
                        Random rand = new Random();
                        long   tid  = Thread.currentThread().getId();


                        byte[] data = new byte[10];//(tt + 1) * 1000];
                        Arrays.fill(data, (byte) 0);

                        for(int i = 0; i < 100; i++)
                        {
                            if(addDocumentToJcr(tid, i, data) == null)
                            {
                                break;
                            }
                            try
                            {
                                Thread.sleep(rand.nextInt(500));
                            }
                            catch(InterruptedException e)
                            {
                                log.error("Sleep interrupted");
                            }

                        }
                    }
                });
        }

        for(Thread thread : threads)
        {
            thread.start();

            try
            {
                Thread.sleep(1000);
            }
            catch(InterruptedException e)
            {
                log.error("Problem with sleep");
            }
        }

        for(Thread thread : threads)
        {
            try
            {
                thread.join();
            }
            catch(InterruptedException e)
            {
                log.error("Problem joining with thread " + thread.getId());
            }
        }

        try
        {
            engine.shutdownAndAwaitTermination(5, TimeUnit.SECONDS);
        }
        catch(InterruptedException e)
        {
            log.error("Timeout waiting to terminate JCR engine");
        }

        log.info("Exiting");
    }

    public static String addDocumentToJcr(long   tid,
                                          int    n,
                                          byte[] value)
    {
        log.debug("{" + tid + "}[" + n + "] Add a new document");

        try
        {
            //TransactionManager txnm = new JBossStandaloneJTAManagerLookup().getTransactionManager();

            Session              session = null;
            ByteArrayInputStream bis     = null;

            try
            {
                session = repository.login();
                //txnm.begin();

                Node rootNode = session.getRootNode();

                // Store content
                Node docGroupNode = rootNode.addNode("inf:documentGroup", DOCUMENT_GROUP_TYPE);
                Node doc          = docGroupNode.addNode("inf:documents", DOCUMENT_TYPE);
                doc.setProperty("inf:type", "type/type");

                Node content     = doc.addNode("inf:contents", CONTENT_TYPE);
                Node contentData = content.addNode("inf:contentData", CONTENT_DATA_TYPE);
                Node file        = contentData.addNode("jcr:content", RESOURCE_DATA_TYPE);
                bis = new ByteArrayInputStream(value);

                Binary b = session.getValueFactory().createBinary(bis);
                file.setProperty("jcr:data", b);

                session.save();
                //txnm.commit();

                session.logout();
                session = null;

                log.debug("{" + tid + "}[" + n + "] Added a new document");

                return docGroupNode.getIdentifier();
            }
            catch(InvalidItemStateException e)
            {
                //txnm.rollback();
                log.error("{" + tid + "}[" + n + "] InvalidItemState when creating new document", e);
            }
            catch(RepositoryException e)
            {
                //txnm.rollback();
                log.error("{" + tid + "}[" + n + "] Error creating new document", e);
            }
            finally
            {
                if(bis != null)
                {
                    try
                    {
                        bis.close();
                    }
                    catch(IOException e)
                    {
                        log.warn("{" + tid + "}[" + n + "] Failed to close ByteArrayInputStream", e);
                    }
                }

                if(session != null)
                {
                    session.logout();
                    session = null;
                }
            }
        }
        catch(Exception e)
        {
            log.error("Problem with TransactionManager", e);
        }

        return null;
    }
}